@@ -14,6 +14,7 @@
 
 from . import backends
 from .. import build
+from .. import dependencies
 from .. import mesonlib
 import uuid, os, sys
 
@@ -27,21 +28,22 @@ class XCodeBackend(backends.Backend):
         self.project_conflist = self.gen_id()
         self.indent = '       '
         self.indent_level = 0
-        self.xcodetypemap = {'c' : 'sourcecode.c.c',
-                             'a' : 'archive.ar',
+        self.xcodetypemap = {'c': 'sourcecode.c.c',
+                             'a': 'archive.ar',
                              'cc': 'sourcecode.cpp.cpp',
-                             'cxx' : 'sourcecode.cpp.cpp',
-                             'cpp' : 'sourcecode.cpp.cpp',
-                             'c++' : 'sourcecode.cpp.cpp',
-                             'm' : 'sourcecode.c.objc',
-                             'mm' : 'sourcecode.cpp.objcpp',
-                             'h' : 'sourcecode.c.h',
-                             'hpp' : 'sourcecode.cpp.h',
-                             'hxx' : 'sourcecode.cpp.h',
-                             'hh' : 'sourcecode.cpp.hh',
-                             'inc' : 'sourcecode.c.h',
-                             'dylib' : 'compiled.mach-o.dylib',
-                             'o' : 'compiled.mach-o.objfile',}
+                             'cxx': 'sourcecode.cpp.cpp',
+                             'cpp': 'sourcecode.cpp.cpp',
+                             'c++': 'sourcecode.cpp.cpp',
+                             'm': 'sourcecode.c.objc',
+                             'mm': 'sourcecode.cpp.objcpp',
+                             'h': 'sourcecode.c.h',
+                             'hpp': 'sourcecode.cpp.h',
+                             'hxx': 'sourcecode.cpp.h',
+                             'hh': 'sourcecode.cpp.hh',
+                             'inc': 'sourcecode.c.h',
+                             'dylib': 'compiled.mach-o.dylib',
+                             'o': 'compiled.mach-o.objfile',
+                             }
         self.maingroup_id = self.gen_id()
         self.all_id = self.gen_id()
         self.all_buildconf_id = self.gen_id()
@@ -59,13 +61,13 @@ class XCodeBackend(backends.Backend):
         return dirname
 
     def write_line(self, text):
-        self.ofile.write(self.indent*self.indent_level + text)
+        self.ofile.write(self.indent * self.indent_level + text)
         if not text.endswith('\n'):
             self.ofile.write('\n')
 
     def generate(self, interp):
         self.interpreter = interp
-        test_data = self.serialise_tests()[0]
+        test_data = self.serialize_tests()[0]
         self.generate_filemap()
         self.generate_buildmap()
         self.generate_buildstylemap()
@@ -76,6 +78,7 @@ class XCodeBackend(backends.Backend):
         self.generate_buildall_configurations_map()
         self.generate_test_configurations_map()
         self.generate_native_target_map()
+        self.generate_native_frameworks_map()
         self.generate_source_phase_map()
         self.generate_target_dependency_map()
         self.generate_pbxdep_map()
@@ -90,6 +93,7 @@ class XCodeBackend(backends.Backend):
             self.generate_pbx_build_style()
             self.generate_pbx_container_item_proxy()
             self.generate_pbx_file_reference()
+            self.generate_pbx_frameworks_buildphase()
             self.generate_pbx_group()
             self.generate_pbx_native_target()
             self.generate_pbx_project()
@@ -129,27 +133,32 @@ class XCodeBackend(backends.Backend):
                     self.buildmap[o] = self.gen_id()
 
     def generate_buildstylemap(self):
-        self.buildstylemap = {'debug' : self.gen_id()}
+        self.buildstylemap = {'debug': self.gen_id()}
 
     def generate_build_phase_map(self):
-        self.buildphasemap = {}
-        for t in self.build.targets:
-            self.buildphasemap[t] = self.gen_id()
+        for tname, t in self.build.targets.items():
+            # generate id for our own target-name
+            t.buildphasemap = {}
+            t.buildphasemap[tname] = self.gen_id()
+            # each target can have it's own Frameworks/Sources/..., generate id's for those
+            t.buildphasemap['Frameworks'] = self.gen_id()
+            t.buildphasemap['Resources'] = self.gen_id()
+            t.buildphasemap['Sources'] = self.gen_id()
 
     def generate_build_configuration_map(self):
         self.buildconfmap = {}
         for t in self.build.targets:
-            bconfs = {'debug' : self.gen_id()}
+            bconfs = {'debug': self.gen_id()}
             self.buildconfmap[t] = bconfs
 
     def generate_project_configurations_map(self):
-        self.project_configurations = {'debug' : self.gen_id()}
+        self.project_configurations = {'debug': self.gen_id()}
 
     def generate_buildall_configurations_map(self):
-        self.buildall_configurations = {'debug' : self.gen_id()}
+        self.buildall_configurations = {'debug': self.gen_id()}
 
     def generate_test_configurations_map(self):
-        self.test_configurations = {'debug' : self.gen_id()}
+        self.test_configurations = {'debug': self.gen_id()}
 
     def generate_build_configurationlist_map(self):
         self.buildconflistmap = {}
@@ -161,6 +170,16 @@ class XCodeBackend(backends.Backend):
         for t in self.build.targets:
             self.native_targets[t] = self.gen_id()
 
+    def generate_native_frameworks_map(self):
+        self.native_frameworks = {}
+        self.native_frameworks_fileref = {}
+        for t in self.build.targets.values():
+            for dep in t.get_external_deps():
+                if isinstance(dep, dependencies.AppleFrameworks):
+                    for f in dep.frameworks:
+                        self.native_frameworks[f] = self.gen_id()
+                        self.native_frameworks_fileref[f] = self.gen_id()
+
     def generate_target_dependency_map(self):
         self.target_dependency_map = {}
         for tname, t in self.build.targets.items():
@@ -185,35 +204,35 @@ class XCodeBackend(backends.Backend):
     def generate_pbx_aggregate_target(self):
         self.ofile.write('\n/* Begin PBXAggregateTarget section */\n')
         self.write_line('%s /* ALL_BUILD */ = {' % self.all_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXAggregateTarget;')
         self.write_line('buildConfigurationList = %s;' % self.all_buildconf_id)
         self.write_line('buildPhases = (')
         self.write_line(');')
         self.write_line('dependencies = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for t in self.build.targets:
             self.write_line('%s /* PBXTargetDependency */,' % self.pbx_dep_map[t])
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('name = ALL_BUILD;')
         self.write_line('productName = ALL_BUILD;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.write_line('%s /* RUN_TESTS */ = {' % self.test_id)
-        self.indent_level +=1
+        self.indent_level += 1
         self.write_line('isa = PBXAggregateTarget;')
         self.write_line('buildConfigurationList = %s;' % self.test_buildconf_id)
         self.write_line('buildPhases = (')
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('%s /* test run command */,' % self.test_command_id)
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('dependencies = (')
         self.write_line(');')
         self.write_line('name = RUN_TESTS;')
         self.write_line('productName = RUN_TESTS;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.ofile.write('/* End PBXAggregateTarget section */\n')
 
@@ -221,7 +240,14 @@ class XCodeBackend(backends.Backend):
         self.ofile.write('\n/* Begin PBXBuildFile section */\n')
         templ = '%s /* %s */ = { isa = PBXBuildFile; fileRef = %s /* %s */; settings = { COMPILER_FLAGS = "%s"; }; };\n'
         otempl = '%s /* %s */ = { isa = PBXBuildFile; fileRef = %s /* %s */;};\n'
+
         for t in self.build.targets.values():
+
+            for dep in t.get_external_deps():
+                if isinstance(dep, dependencies.AppleFrameworks):
+                    for f in dep.frameworks:
+                        self.ofile.write('%s /* %s.framework in Frameworks */ = {isa = PBXBuildFile; fileRef = %s /* %s.framework */; };\n' % (self.native_frameworks[f], f, self.native_frameworks_fileref[f], f))
+
             for s in t.sources:
                 if isinstance(s, mesonlib.File):
                     s = s.fname
@@ -269,12 +295,18 @@ class XCodeBackend(backends.Backend):
             self.write_line('proxyType = 1;')
             self.write_line('remoteGlobalIDString = %s;' % self.native_targets[t])
             self.write_line('remoteInfo = "%s";' % t)
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXContainerItemProxy section */\n')
 
     def generate_pbx_file_reference(self):
         self.ofile.write('\n/* Begin PBXFileReference section */\n')
+
+        for t in self.build.targets.values():
+            for dep in t.get_external_deps():
+                if isinstance(dep, dependencies.AppleFrameworks):
+                    for f in dep.frameworks:
+                        self.ofile.write('%s /* %s.framework */ = {isa = PBXFileReference; lastKnownFileType = wrapper.framework; name = %s.framework; path = System/Library/Frameworks/%s.framework; sourceTree = SDKROOT; };\n' % (self.native_frameworks_fileref[f], f, f, f))
         src_templ = '%s /* %s */ = { isa = PBXFileReference; explicitFileType = "%s"; fileEncoding = 4; name = "%s"; path = "%s"; sourceTree = SOURCE_ROOT; };\n'
         for fname, idval in self.filemap.items():
             fullpath = os.path.join(self.environment.get_source_dir(), fname)
@@ -299,6 +331,27 @@ class XCodeBackend(backends.Backend):
             self.ofile.write(target_templ % (idval, tname, typestr, path, reftype))
         self.ofile.write('/* End PBXFileReference section */\n')
 
+    def generate_pbx_frameworks_buildphase(self):
+        for tname, t in self.build.targets.items():
+            self.ofile.write('\n/* Begin PBXFrameworksBuildPhase section */\n')
+            self.indent_level += 1
+            self.write_line('%s /* %s */ = {\n' % (t.buildphasemap['Frameworks'], 'Frameworks'))
+            self.indent_level += 1
+            self.write_line('isa = PBXFrameworksBuildPhase;\n')
+            self.write_line('buildActionMask = %s;\n' % (2147483647))
+            self.write_line('files = (\n')
+            self.indent_level += 1
+            for dep in t.get_external_deps():
+                if isinstance(dep, dependencies.AppleFrameworks):
+                    for f in dep.frameworks:
+                        self.write_line('%s /* %s.framework in Frameworks */,\n' % (self.native_frameworks[f], f))
+            self.indent_level -= 1
+            self.write_line(');\n')
+            self.write_line('runOnlyForDeploymentPostprocessing = 0;\n')
+            self.indent_level -= 1
+            self.write_line('};\n')
+        self.ofile.write('/* End PBXFrameworksBuildPhase section */\n')
+
     def generate_pbx_group(self):
         groupmap = {}
         target_src_map = {}
@@ -309,15 +362,17 @@ class XCodeBackend(backends.Backend):
         sources_id = self.gen_id()
         resources_id = self.gen_id()
         products_id = self.gen_id()
+        frameworks_id = self.gen_id()
         self.write_line('%s = {' % self.maingroup_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('%s /* Sources */,' % sources_id)
         self.write_line('%s /* Resources */,' % resources_id)
         self.write_line('%s /* Products */,' % products_id)
-        self.indent_level-=1
+        self.write_line('%s /* Frameworks */,' % frameworks_id)
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('sourceTree = "<group>";')
         self.indent_level -= 1
@@ -325,48 +380,68 @@ class XCodeBackend(backends.Backend):
 
         # Sources
         self.write_line('%s /* Sources */ = {' % sources_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for t in self.build.targets:
             self.write_line('%s /* %s */,' % (groupmap[t], t))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('name = Sources;')
         self.write_line('sourcetree = "<group>";')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         self.write_line('%s /* Resources */ = {' % resources_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
         self.write_line(');')
         self.write_line('name = Resources;')
         self.write_line('sourceTree = "<group>";')
-        self.indent_level-=1
+        self.indent_level -= 1
+        self.write_line('};')
+
+        self.write_line('%s /* Frameworks */ = {' % frameworks_id)
+        self.indent_level += 1
+        self.write_line('isa = PBXGroup;')
+        self.write_line('children = (')
+        # write frameworks
+        self.indent_level += 1
+
+        for t in self.build.targets.values():
+            for dep in t.get_external_deps():
+                if isinstance(dep, dependencies.AppleFrameworks):
+                    for f in dep.frameworks:
+                        self.write_line('%s /* %s.framework */,\n' % (self.native_frameworks_fileref[f], f))
+
+        self.indent_level -= 1
+        self.write_line(');')
+        self.write_line('name = Frameworks;')
+        self.write_line('sourceTree = "<group>";')
+        self.indent_level -= 1
         self.write_line('};')
 
         # Targets
         for t in self.build.targets:
             self.write_line('%s /* %s */ = {' % (groupmap[t], t))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = PBXGroup;')
             self.write_line('children = (')
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('%s /* Source files */,' % target_src_map[t])
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line(');')
             self.write_line('name = "%s";' % t)
             self.write_line('sourceTree = "<group>";')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
             self.write_line('%s /* Source files */ = {' % target_src_map[t])
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = PBXGroup;')
             self.write_line('children = (')
-            self.indent_level+=1
+            self.indent_level += 1
             for s in self.build.targets[t].sources:
                 s = os.path.join(s.subdir, s.fname)
                 if isinstance(s, str):
@@ -374,26 +449,26 @@ class XCodeBackend(backends.Backend):
             for o in self.build.targets[t].objects:
                 o = os.path.join(self.build.targets[t].subdir, o)
                 self.write_line('%s /* %s */,' % (self.filemap[o], o))
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line(');')
             self.write_line('name = "Source files";')
             self.write_line('sourceTree = "<group>";')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
 
         # And finally products
         self.write_line('%s /* Products */ = {' % products_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = PBXGroup;')
         self.write_line('children = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for t in self.build.targets:
             self.write_line('%s /* %s */,' % (self.target_filemap[t], t))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('name = Products;')
         self.write_line('sourceTree = "<group>";')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.ofile.write('/* End PBXGroup section */\n')
 
@@ -402,25 +477,26 @@ class XCodeBackend(backends.Backend):
         for tname, idval in self.native_targets.items():
             t = self.build.targets[tname]
             self.write_line('%s /* %s */ = {' % (idval, tname))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = PBXNativeTarget;')
-            self.write_line('buildConfigurationList = %s /* Build configuration list for PBXNativeTarget "%s" */;'\
+            self.write_line('buildConfigurationList = %s /* Build configuration list for PBXNativeTarget "%s" */;'
                             % (self.buildconflistmap[tname], tname))
             self.write_line('buildPhases = (')
-            self.indent_level+=1
-            self.write_line('%s /* Sources */,' % self.buildphasemap[tname])
-            self.indent_level-=1
+            self.indent_level += 1
+            for bpname, bpval in t.buildphasemap.items():
+                self.write_line('%s /* %s yyy */,' % (bpval, bpname))
+            self.indent_level -= 1
             self.write_line(');')
             self.write_line('buildRules = (')
             self.write_line(');')
             self.write_line('dependencies = (')
-            self.indent_level+=1
+            self.indent_level += 1
             for lt in self.build.targets[tname].link_targets:
                 # NOT DOCUMENTED, may need to make different links
                 # to same target have different targetdependency item.
                 idval = self.pbx_dep_map[lt.get_id()]
                 self.write_line('%s /* PBXTargetDependency */,' % idval)
-            self.indent_level -=1
+            self.indent_level -= 1
             self.write_line(");")
             self.write_line('name = "%s";' % tname)
             self.write_line('productName = "%s";' % tname)
@@ -434,7 +510,7 @@ class XCodeBackend(backends.Backend):
             else:
                 raise MesonException('Unknown target type for %s' % tname)
             self.write_line('productType = "%s";' % typestr)
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXNativeTarget section */\n')
 
@@ -495,27 +571,28 @@ class XCodeBackend(backends.Backend):
         cmdstr = ' '.join(["'%s'" % i for i in cmd])
         self.write_line('shellScript = "%s";' % cmdstr)
         self.write_line('showEnvVarsInLog = 0;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
         self.ofile.write('/* End PBXShellScriptBuildPhase section */\n')
 
     def generate_pbx_sources_build_phase(self):
         self.ofile.write('\n/* Begin PBXSourcesBuildPhase section */\n')
         for name, phase_id in self.source_phase.items():
-            self.write_line('%s /* Sources */ = {' % self.buildphasemap[name])
-            self.indent_level+=1
+            t = self.build.targets[name]
+            self.write_line('%s /* Sources */ = {' % (t.buildphasemap[name]))
+            self.indent_level += 1
             self.write_line('isa = PBXSourcesBuildPhase;')
             self.write_line('buildActionMask = 2147483647;')
             self.write_line('files = (')
-            self.indent_level+=1
+            self.indent_level += 1
             for s in self.build.targets[name].sources:
                 s = os.path.join(s.subdir, s.fname)
                 if not self.environment.is_header(s):
                     self.write_line('%s /* %s */,' % (self.buildmap[s], os.path.join(self.environment.get_source_dir(), s)))
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line(');')
             self.write_line('runOnlyForDeploymentPostprocessing = 0;')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXSourcesBuildPhase section */\n')
 
@@ -528,7 +605,7 @@ class XCodeBackend(backends.Backend):
             self.write_line('isa = PBXTargetDependency;')
             self.write_line('target = %s /* %s */;' % (self.native_targets[t], t))
             self.write_line('targetProxy = %s /* PBXContainerItemProxy */;' % self.containerproxy_map[t])
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
         self.ofile.write('/* End PBXTargetDependency section */\n')
 
@@ -537,24 +614,24 @@ class XCodeBackend(backends.Backend):
         # First the setup for the toplevel project.
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */ = {' % (self.project_configurations[buildtype], buildtype))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = XCBuildConfiguration;')
             self.write_line('buildSettings = {')
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('ARCHS = "$(ARCHS_STANDARD_32_64_BIT)";')
             self.write_line('ONLY_ACTIVE_ARCH = YES;')
             self.write_line('SDKROOT = "macosx";')
             self.write_line('SYMROOT = "%s/build";' % self.environment.get_build_dir())
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
             self.write_line('name = "%s";' % buildtype)
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
 
         # Then the all target.
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */ = {' % (self.buildall_configurations[buildtype], buildtype))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = XCBuildConfiguration;')
             self.write_line('buildSettings = {')
             self.indent_level += 1
@@ -573,16 +650,16 @@ class XCodeBackend(backends.Backend):
             self.write_line('SYMROOT = "%s";' % self.environment.get_build_dir())
             self.write_line('USE_HEADERMAP = NO;')
             self.write_line('WARNING_CFLAGS = ("-Wmost", "-Wno-four-char-constants", "-Wno-unknown-pragmas", );')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
             self.write_line('name = "%s";' % buildtype)
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
 
         # Then the test target.
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */ = {' % (self.test_configurations[buildtype], buildtype))
-            self.indent_level+=1
+            self.indent_level += 1
             self.write_line('isa = XCBuildConfiguration;')
             self.write_line('buildSettings = {')
             self.indent_level += 1
@@ -601,14 +678,14 @@ class XCodeBackend(backends.Backend):
             self.write_line('SYMROOT = "%s";' % self.environment.get_build_dir())
             self.write_line('USE_HEADERMAP = NO;')
             self.write_line('WARNING_CFLAGS = ("-Wmost", "-Wno-four-char-constants", "-Wno-unknown-pragmas", );')
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
             self.write_line('name = "%s";' % buildtype)
-            self.indent_level-=1
+            self.indent_level -= 1
             self.write_line('};')
 
         # Now finally targets.
-        langnamemap = {'c' : 'C', 'cpp' : 'CPLUSPLUS', 'objc' : 'OBJC', 'objcpp' : 'OBJCPLUSPLUS'}
+        langnamemap = {'c': 'C', 'cpp': 'CPLUSPLUS', 'objc': 'OBJC', 'objcpp': 'OBJCPLUSPLUS'}
         for target_name, target in self.build.targets.items():
             for buildtype in self.buildtypes:
                 dep_libs = []
@@ -653,7 +730,7 @@ class XCodeBackend(backends.Backend):
                         langargs[langnamemap[lang]] = args
                 symroot = os.path.join(self.environment.get_build_dir(), target.subdir)
                 self.write_line('%s /* %s */ = {' % (valid, buildtype))
-                self.indent_level+=1
+                self.indent_level += 1
                 self.write_line('isa = XCBuildConfiguration;')
                 self.write_line('buildSettings = {')
                 self.indent_level += 1
@@ -688,57 +765,57 @@ class XCodeBackend(backends.Backend):
                 self.write_line('SYMROOT = "%s";' % symroot)
                 self.write_line('USE_HEADERMAP = NO;')
                 self.write_line('WARNING_CFLAGS = ("-Wmost", "-Wno-four-char-constants", "-Wno-unknown-pragmas", );')
-                self.indent_level-=1
+                self.indent_level -= 1
                 self.write_line('};')
                 self.write_line('name = "%s";' % buildtype)
-                self.indent_level-=1
+                self.indent_level -= 1
                 self.write_line('};')
         self.ofile.write('/* End XCBuildConfiguration section */\n')
 
     def generate_xc_configurationList(self):
         self.ofile.write('\n/* Begin XCConfigurationList section */\n')
         self.write_line('%s /* Build configuration list for PBXProject "%s" */ = {' % (self.project_conflist, self.build.project_name))
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = XCConfigurationList;')
         self.write_line('buildConfigurations = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */,' % (self.project_configurations[buildtype], buildtype))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('defaultConfigurationIsVisible = 0;')
         self.write_line('defaultConfigurationName = debug;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         # Now the all target
         self.write_line('%s /* Build configuration list for PBXAggregateTarget "ALL_BUILD" */ = {' % self.all_buildconf_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = XCConfigurationList;')
         self.write_line('buildConfigurations = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */,' % (self.buildall_configurations[buildtype], buildtype))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('defaultConfigurationIsVisible = 0;')
         self.write_line('defaultConfigurationName = debug;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         # Test target
         self.write_line('%s /* Build configuration list for PBXAggregateTarget "ALL_BUILD" */ = {' % self.test_buildconf_id)
-        self.indent_level+=1
+        self.indent_level += 1
         self.write_line('isa = XCConfigurationList;')
         self.write_line('buildConfigurations = (')
-        self.indent_level+=1
+        self.indent_level += 1
         for buildtype in self.buildtypes:
             self.write_line('%s /* %s */,' % (self.test_configurations[buildtype], buildtype))
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line(');')
         self.write_line('defaultConfigurationIsVisible = 0;')
         self.write_line('defaultConfigurationName = debug;')
-        self.indent_level-=1
+        self.indent_level -= 1
         self.write_line('};')
 
         for target_name in self.build.targets:
