@@ -264,7 +264,7 @@ def detect_vcs(source_dir):
 
 def grab_leading_numbers(vstr, strict=False):
     result = []
-    for x in vstr.split('.'):
+    for x in vstr.rstrip('.').split('.'):
         try:
             result.append(int(x))
         except ValueError as e:
@@ -716,6 +716,7 @@ def windows_proof_rmtree(f):
 
 def unholder_array(entries):
     result = []
+    entries = flatten(entries)
     for e in entries:
         if hasattr(e, 'held_object'):
             e = e.held_object
