@@ -63,6 +63,14 @@ add_builtin_argument('warnlevel', dest='warning_level')
 add_builtin_argument('stdsplit', action='store_false')
 add_builtin_argument('errorlogs', action='store_false')
 
+def wrapmodetype(string):
+    try:
+        return getattr(WrapMode, string)
+    except AttributeError:
+        msg = ', '.join([t.name.lower() for t in WrapMode])
+        msg = 'invalid argument {!r}, use one of {}'.format(string, msg)
+        raise argparse.ArgumentTypeError(msg)
+
 parser.add_argument('--cross-file', default=None,
                     help='File describing cross compilation environment.')
 parser.add_argument('-D', action='append', dest='projectoptions', default=[], metavar="option",
@@ -71,7 +79,7 @@ parser.add_argument('-v', '--version', action='version',
                     version=coredata.version)
 # See the mesonlib.WrapMode enum for documentation
 parser.add_argument('--wrap-mode', default=WrapMode.default,
-                    type=lambda t: getattr(WrapMode, t), choices=WrapMode,
+                    type=wrapmodetype, choices=WrapMode,
                     help='Special wrap mode to use')
 parser.add_argument('directories', nargs='*')
 
@@ -113,13 +121,14 @@ class MesonApp:
         priv_dir = os.path.join(build_dir, 'meson-private/coredata.dat')
         if os.path.exists(priv_dir):
             if not handshake:
-                msg = '''Trying to run Meson on a build directory that has already been configured.
+                print('''Trying to run Meson on a build directory that has already been configured.
 If you want to build it, just run your build command (e.g. ninja) inside the
 build directory. Meson will autodetect any changes in your setup and regenerate
-itself as required.
+itself as required. Though it shouldn't be necessary, running ninja reconfigure
+will force Meson to regenerate the build files.
 
-If you want to change option values, use the mesonconf tool instead.'''
-                raise RuntimeError(msg)
+If you want to change option values, use meson configure instead.''')
+                sys.exit(0)
         else:
             if handshake:
                 raise RuntimeError('Something went terribly wrong. Please file a bug.')
@@ -135,6 +144,12 @@ If you want to change option values, use the mesonconf tool instead.'''
     def generate(self):
         env = environment.Environment(self.source_dir, self.build_dir, self.meson_script_launcher, self.options, self.original_cmd_line_args)
         mlog.initialize(env.get_log_dir())
+        try:
+            self._generate(env)
+        finally:
+            mlog.shutdown()
+
+    def _generate(self, env):
         mlog.debug('Build started at', datetime.datetime.now().isoformat())
         mlog.debug('Python binary:', sys.executable)
         mlog.debug('Python system:', platform.system())
