@@ -19,6 +19,7 @@ from .. import mesonlib
 from .. import mlog
 from . import ModuleReturnValue
 from . import ExtensionModule
+from . import permittedKwargs
 
 
 class PkgConfigModule(ExtensionModule):
@@ -43,7 +44,7 @@ class PkgConfigModule(ExtensionModule):
 
     def generate_pkgconfig_file(self, state, libraries, subdirs, name, description,
                                 url, version, pcfile, pub_reqs, priv_reqs,
-                                conflicts, priv_libs, variables):
+                                conflicts, priv_libs, extra_cflags, variables):
         coredata = state.environment.get_coredata()
         outdir = state.environment.scratch_dir
         fname = os.path.join(outdir, pcfile)
@@ -101,6 +102,9 @@ class PkgConfigModule(ExtensionModule):
                     ofile.write('-I${includedir}')
                 else:
                     ofile.write(os.path.join('-I${includedir}', h))
+            for f in extra_cflags:
+                ofile.write(' ')
+                ofile.write(f)
             ofile.write('\n')
 
     def process_libs(self, libs):
@@ -115,6 +119,9 @@ class PkgConfigModule(ExtensionModule):
             processed_libs.append(l)
         return processed_libs
 
+    @permittedKwargs({'libraries', 'version', 'name', 'description', 'filebase',
+                      'subdirs', 'requires', 'requires_private', 'libraries_private',
+                      'install_dir', 'extra_cflags', 'variables'})
     def generate(self, state, args, kwargs):
         if len(args) > 0:
             raise mesonlib.MesonException('Pkgconfig_gen takes no positional arguments.')
@@ -139,6 +146,7 @@ class PkgConfigModule(ExtensionModule):
         pub_reqs = mesonlib.stringlistify(kwargs.get('requires', []))
         priv_reqs = mesonlib.stringlistify(kwargs.get('requires_private', []))
         conflicts = mesonlib.stringlistify(kwargs.get('conflicts', []))
+        extra_cflags = mesonlib.stringlistify(kwargs.get('extra_cflags', []))
 
         def parse_variable_list(stringlist):
             reserved = ['prefix', 'libdir', 'includedir']
@@ -147,11 +155,11 @@ class PkgConfigModule(ExtensionModule):
                 # foo=bar=baz is ('foo', 'bar=baz')
                 l = var.split('=', 1)
                 if len(l) < 2:
-                    raise mesonlib.MesonException('Variables must be in \'name=value\' format')
+                    raise mesonlib.MesonException('Invalid variable "{}". Variables must be in \'name=value\' format'.format(var))
 
                 name, value = l[0].strip(), l[1].strip()
                 if not name or not value:
-                    raise mesonlib.MesonException('Variables must be in \'name=value\' format')
+                    raise mesonlib.MesonException('Invalid variable "{}". Variables must be in \'name=value\' format'.format(var))
 
                 # Variable names must not contain whitespaces
                 if any(c.isspace() for c in name):
@@ -174,7 +182,7 @@ class PkgConfigModule(ExtensionModule):
             raise mesonlib.MesonException('Install_dir must be a string.')
         self.generate_pkgconfig_file(state, libs, subdirs, name, description, url,
                                      version, pcfile, pub_reqs, priv_reqs,
-                                     conflicts, priv_libs, variables)
+                                     conflicts, priv_libs, extra_cflags, variables)
         res = build.Data(mesonlib.File(True, state.environment.get_scratch_dir(), pcfile), pkgroot)
         return ModuleReturnValue(res, [res])
 
